#![feature(concat_idents)]
mod graph_parser;
mod rust_bfs;

use graph_parser::*;

use hercules_rt::{runner, HerculesImmBox, HerculesImmBoxTo, HerculesMutBox};

use clap::Parser;

juno_build::juno!("bfs");

#[derive(Parser)]
#[clap(author, version, about, long_about = None)]
struct BFSInputs {
    input: String,
}

fn run_bfs(nodes: &[Node], source: u32, edges: &[u32]) -> Vec<i32> {
    let n = nodes.len() as u64;
    let m = edges.len() as u64;

    let nodes = HerculesImmBox::from(nodes);
    let edges = HerculesImmBox::from(edges);

    let mut runner = runner!(bfs);

    HerculesMutBox::from(async_std::task::block_on(async {
        runner.run(n, m, nodes.to(), source, edges.to()).await
    }))
    .as_slice()
    .to_vec()
}

fn bfs_harness(args: BFSInputs) {
    let BFSInputs { input } = args;

    let (nodes, source, edges) = parse_graph(input);

    let costs_juno = run_bfs(&nodes, source, &edges);
    let costs_ref = rust_bfs::bfs(&nodes, source, &edges);

    assert_eq!(costs_juno, costs_ref);
}

fn main() {
    let args = BFSInputs::parse();
    bfs_harness(args);
}

#[test]
fn bfs_test_4096() {
    bfs_harness(BFSInputs {
        input: "data/graph4096.txt".to_string(),
    });
}

#[test]
fn bfs_test_65536() {
    bfs_harness(BFSInputs {
        input: "data/graph65536.txt".to_string(),
    });
}
